<?php
namespace Lpc\LpcSermons\Controller;

use TYPO3\CMS\Core\Messaging\FlashMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;

class MigrationController extends ActionController
{

	/**
	 * @var \Lpc\LpcSermons\Service\TypoScriptService
	 * @inject
	 */
	protected $typoScriptService;

	/**
	 * sermonRepository
	 *
	 * @var \Lpc\LpcSermons\Domain\Repository\SermonRepository
	 * @inject
	 */
	protected $sermonRepository;

	/**
	 * podcastRepository
	 *
	 * @var \Lpc\LpcSermons\Domain\Repository\PodcastRepository
	 * @inject
	 */
	protected $podcastRepository;

	/**
	 * persistenceManager
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager
	 * @inject
	 */
	protected $persistenceManager;

	/**
	 * @var \Lpc\LpcSermons\Utility\Db
	 */
	protected $externalDb;

	/**
	 * Backend-Session
	 *
	 * @var \Lpc\LpcSermons\Domain\Session\BackendSessionHandler
	 * @inject
	 */
	protected $backendSession;

	/**
	 * contains all created podcasts
	 * @var array
	 */
	protected $podcastCreated = array();

	/**
	 * @var \Lpc\LpcSermons\Utility\FileUtility
	 * @inject
	 */
	protected $fileUtility = null;

	/**
	 * @var string
	 */
	protected $log = '';

	protected $doRawUrlDecode = false;


	/**
	 *
	 */
	public function initializeAction() {
		/** @var \TYPO3\CMS\Extbase\Persistence\Generic\Typo3QuerySettings $querySettings */
		$querySettings = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\Typo3QuerySettings');
        $querySettings->setRespectStoragePage(FALSE);
       	$this->sermonRepository->setDefaultQuerySettings($querySettings);

		$this->backendSession->setStorageKey('tx_lpcsermons');
	}


	public function configAction() {

		$formdata = $this->backendSession->get('sermons_migration_formdata');

		/*
		// test data
		$formdata =
			array (
				'action' => 'run',
				'controller' => 'Migration',
				'srcHost' => 'dedi3190.your-server.de',
				'srcDb' => 'vinbern_typo3',
				'srcDbUsername' => 'vinbern_typo_r',
				'srcDbPw' => 'r3YX1d1t',
				'srcPIDs' => '1197
1092
1166',
				'srcPaths' => 'fileadmin/user_upload/dateien/audio/predigt_1700-15/
fileadmin/user_upload/dateien/audio/predigt_1700-14
fileadmin/user_upload/dateien/audio/input_pfingstkonferenz14/DE-audio/',
			'tarPIDs' => '46
47
48',
				'tarPaths' => 'tx_lpcsermons/migration/predigt_1700-15/
/tx_lpcsermons/migration/predigt_1700-14
/tx_lpcsermons/migration/input_pfingstkonferenz14/DE-audio/
',
			);
		*/

		$this->view->assign('formdata', $formdata);
	}

	public function runAction() {

		set_time_limit(1800);

		// save all arguments to session
		$this->backendSession->store('sermons_migration_formdata', $this->request->getArguments());

		$this->dbConnect();

		// read multiple values (one per line)
		$srcPIDs = $this->getMultipleValue('srcPIDs');
		$srcPaths = $this->getMultipleValue('srcPaths');
		$tarPIDs = $this->getMultipleValue('tarPIDs');
		$tarPaths = $this->getMultipleValue('tarPaths');

		if (intval($this->request->getArgument('doRawUrlDecode')) == 1) {
			$this->doRawUrlDecode = true;
		}

		// all arrays must have the same length
		if (!(count($srcPIDs) == count($srcPaths) && count($tarPIDs) == count($tarPaths) && count($srcPaths) == count($tarPIDs))) {
			$this->addFlashMessage('Error in Configuration. Not all multiline textareas are matching.', 'Error multiline fields', FlashMessage::ERROR);
		}

		for ($i = 0; $i < count($srcPIDs); $i++) {
			$srcPID = $srcPIDs[$i];
			$tarPID = $tarPIDs[$i];

			$srcPath = $srcPaths[$i];
			if ($srcPath !== '-') {
				$srcPath = '/'.$srcPaths[$i].'/';

				// convert paths in default path
				$srcPath = str_replace('//', '/', $srcPath);
				$srcPath = substr($srcPath, 1); // cut first slash
			}

			$tarPath = '/'.$tarPaths[$i].'/';
			$tarPath = str_replace('//', '/', $tarPath);

			$this->importPodcasts($srcPID, $tarPID, $srcPath, $tarPath);
			$this->importSermons($srcPID, $tarPID, $srcPath, $tarPath);
		}

		$this->view->assign('logInformation', $this->log);

	}

	/**
	 * Connect to db
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException
	 */
	protected function dbConnect() {
		// connect to db
		$host = $this->request->getArgument('srcHost');
		$db = $this->request->getArgument('srcDb');
		$username = $this->request->getArgument('srcDbUsername');
		$password = $this->request->getArgument('srcDbPw');
		$this->externalDb = $this->objectManager->get('Lpc\LpcSermons\Utility\Db');
		$this->externalDb->connect($host, $db, $username, $password);
	}

	/**
	 * get the values splitted by newline
	 * @param $varName
	 * @return array
	 * @throws \TYPO3\CMS\Extbase\Mvc\Exception\NoSuchArgumentException
	 */
	protected function getMultipleValue($varName) {
		$varValue = trim($this->request->getArgument($varName));
		$varValueArray = explode("\r\n", $varValue);
		// remove spaces (maybe \r, \n)
		$varValueArray = array_filter($varValueArray, 'trim');
		// remove empty entries
		$varValueArray = array_filter($varValueArray);
		// reoder index
		$varValueArray = array_values($varValueArray);
		return $varValueArray;
	}

	/**
	 * Imports the podcasts
	 * @param $srcPID
	 * @param $tarPID
	 * @param $srcPath
	 * @param $tarPath
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException
	 */
	protected function importPodcasts($srcPID, $tarPID, $srcPath, $tarPath) {
		// import podcasts
		$pdo = $this->externalDb->exec('SELECT * FROM tx_koolsermons_podcasts WHERE pid = :pid AND deleted = 0', array(':pid' => intval($srcPID)));
		$podcasts = $pdo->fetchAll(\PDO::FETCH_ASSOC);
		$fileImportCounter = 0;

		foreach ($podcasts as $podcastOld) {
			/** @var \Lpc\LpcSermons\Domain\Model\Podcast $podcastNew */
			$podcastNew = $this->objectManager->get('\Lpc\LpcSermons\Domain\Model\Podcast');

			// mapping
			// 	meta
			$podcastNew->setPid($tarPID);
			$podcastNew->setAdditionalInfo($podcastOld['uid']);
			$podcastNew->setCrdate($podcastOld['crdate']);
			$podcastNew->setHidden($podcastOld['hidden']);

			// 	data
			$podcastNew->setTitle($podcastOld['title']);
			$podcastNew->setSubtitle($podcastOld['subtitle']);
			$podcastNew->setDescription($podcastOld['description']);
			$podcastNew->setLanguage($podcastOld['language']);
			$podcastNew->setCopyright($podcastOld['copyright']);
			$podcastNew->setEditorName($podcastOld['editorName']);
			$podcastNew->setEditorEmail($podcastOld['editorEmail']);
			$podcastNew->setAuthor($podcastOld['author']);
			$podcastNew->setExplicit($podcastOld['explicit']);
			$podcastNew->setCategory($podcastOld['category']);
			$podcastNew->setXmlFile($podcastOld['xml_file']);

			$this->podcastRepository->add($podcastNew);

			// save all podcasts -> to get ids
			$this->persistenceManager->persistAll();

			// save for later
			$this->podcastCreated[$podcastOld['uid']] = $podcastNew;

			//  add image
			foreach (array('image' => 'image') as $oldName => $newName) {
				$success = $this->fileUtility->handleFileMigration(
					$srcPath,
					$podcastOld[$oldName],
					$tarPath,
					$podcastNew->getUid(),
					$podcastNew->getPid(),
					'tx_lpcsermons_domain_model_podcast',
					$newName
				);
				if (!is_null($success)) {
					if ($success === true) {
						// count
						$fileImportCounter++;
					} else {
						$this->log .= 'Podcasts resources error: "'.$podcastNew->getTitle().'" (uid: '.$podcastNew->getUid().'): <br>'.$success.'<br><br>';
					}
				}
			}
		}

		$this->log .= 'Podcasts imported: '.count($this->podcastCreated).' with '.$fileImportCounter.' new Files.<br><br><hr><br><br>';
	}

	/**
	 * Imports the sermons
	 * @param $srcPID
	 * @param $tarPID
	 * @param $srcPath
	 * @param $tarPath
	 * @throws \TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException
	 */
	protected function importSermons($srcPID, $tarPID, $srcPath, $tarPath) {
		// import sermons
		$pdo = $this->externalDb->exec('SELECT * FROM tx_koolsermons_sermons WHERE pid = :pid AND deleted = 0 ORDER BY date ASC', array(':pid' => intval($srcPID)));
		$sermons = $pdo->fetchAll(\PDO::FETCH_ASSOC);
		$fileImportCounter = 0;

		foreach ($sermons as $sermonOld) {
			/** @var \Lpc\LpcSermons\Domain\Model\Sermon $sermonNew */
			$sermonNew = $this->objectManager->get('\Lpc\LpcSermons\Domain\Model\Sermon');

			// mapping
			// 	meta
			$sermonNew->setPid($tarPID);
			$sermonNew->setCrdate($sermonOld['crdate']);
			$sermonNew->setStarttime($sermonOld['starttime']);
			$sermonNew->setEndtime($sermonOld['endtime']);

			$sermonNew->setHidden($sermonOld['hidden']);
			if (intval($sermonOld['podcast_id']) > 0) {
				$podcastId = intval($sermonOld['podcast_id']);
				if (isset($this->podcastCreated[$podcastId])) {
					$sermonNew->setPodcast($this->podcastCreated[$podcastId]);
				}
			}

			// 	data
			$sermonNew->setFeGroup($sermonOld['fe_group']);
			$sermonNew->setKoolEventId($sermonOld['kool_event_id']);
			$sermonNew->setTitle($sermonOld['title']);
			$sermonNew->setPreacher($sermonOld['speaker']);
			$sermonNew->setDate($sermonOld['date']);
			$sermonNew->setDescription($sermonOld['description']);
			$sermonNew->setLanguage($sermonOld['language']);
			$sermonNew->setReference($sermonOld['scripture']);
			$sermonNew->setUrl($sermonOld['link']);

			$this->sermonRepository->add($sermonNew);
			$this->persistenceManager->persistAll();

			// 	files (old col name => new col name)
			foreach (array('mp3file' => 'mp3file', 'resources' => 'resources', 'podcastfile' => 'podcastfile') as $oldName => $newName) {

				// needed in special cases
				$srcPathFile = $sermonOld[$oldName];
				if ($this->doRawUrlDecode) {
					$srcPathFile = rawurldecode($srcPathFile);
				}

				$success = $this->fileUtility->handleFileMigration(
					$srcPath,
					$srcPathFile,
					$tarPath,
					$sermonNew->getUid(),
					$sermonNew->getPid(),
					'tx_lpcsermons_domain_model_sermon',
					$newName
				);
				if (!is_null($success)) {
					if ($success === true) {
						// count
						$fileImportCounter++;
					} else {
						$this->log .= 'Sermon resources error: "'.$sermonNew->getTitle().'" (uid: '.$sermonNew->getUid().'): <br>'.$success.'<br><br>';
					}
				}
			}
		}
		// save all podcasts -> to get ids
		$this->persistenceManager->persistAll();

		$this->log .= 'Sermons imported: '.count($sermons).' with '.$fileImportCounter.' new Files.<br><br><hr><br><br>';
	}
}

?>
