<?php
namespace Lpc\LpcSermons\Domain\Model;

	/***************************************************************
	 *
	 *  Copyright notice
	 *
	 *  (c) 2014 Michael Hadorn <michael.hadorn@laupercomputing.ch>, Lauper Computing
	 *
	 *  All rights reserved
	 *
	 *  This script is part of the TYPO3 project. The TYPO3 project is
	 *  free software; you can redistribute it and/or modify
	 *  it under the terms of the GNU General Public License as published by
	 *  the Free Software Foundation; either version 3 of the License, or
	 *  (at your option) any later version.
	 *
	 *  The GNU General Public License can be found at
	 *  http://www.gnu.org/copyleft/gpl.html.
	 *
	 *  This script is distributed in the hope that it will be useful,
	 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
	 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 *  GNU General Public License for more details.
	 *
	 *  This copyright notice MUST APPEAR in all copies of the script!
	 ***************************************************************/
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;

/**
 * Sermon
 */
class Sermon extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

	/**
	 * Titel
	 *
	 * @var string
	 */
	protected $title = '';

	/**
	 * Prediger
	 *
	 * @var string
	 */
	protected $preacher = '';

	/**
	 * Bild
	 *
	 * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
	 */
	protected $image = NULL;

	/**
	 * Datum
	 *
	 * @var \DateTime
	 */
	protected $date = NULL;

	/**
	 * Beschreibung
	 *
	 * @var string
	 */
	protected $description = '';

	/**
	 * Sprache
	 *
	 * @var string
	 */
	protected $language = '';

	/**
	 * Bibelstelle
	 *
	 * @var string
	 */
	protected $reference = '';

	/**
	 * Link (URL)
	 *
	 * @var string
	 */
	protected $url = '';

	/**
	 * mp3file
	 *
	 * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
	 */
	protected $mp3file = NULL;

	/**
	 * podcastfile
	 *
	 * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
	 */
	protected $podcastfile = NULL;

	/**
	 * Video Link
	 *
	 * @var string
	 */
	protected $videourl = '';

	/**
	 * resources
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\FileReference>
	 * @cascade remove
	 */
	protected $resources = NULL;

	/**
	 * Podcast zur Predigt
	 *
	 * @var \Lpc\LpcSermons\Domain\Model\Podcast
	 */
	protected $podcast = NULL;

	/**
	 * cacheTs
	 *
	 * @var int
	 */
	protected $cacheTs = NULL;

	/**
	 * serialized array
	 *
	 * @var string
	 */
	protected $cacheMp3info = NULL;

	/**
	 * timestamp of creation
	 * @var int
	 */
	protected $crdate = 0;


	/**
	 * @var string
	 */
	protected $tstamp = '';

	/**
	 * @var string
	 */
	protected $hidden = '';

	/**
	 * @var string
	 */
	protected $deleted = '';

	/**
	 * @var string
	 */
	protected $cruserId = '';

	/**
	 * @var int
	 */
	protected $feGroup = 0;

	/**
	 * @var int
	 */
	protected $koolEventId = 0;

	/**
	 * @var DateTime
	 */
	protected $starttime = null;

	/**
	 * @var DateTime
	 */
	protected $endtime = null;


	/**
	 * Returns the title
	 *
	 * @return string $title
	 */
	public function getTitle()
	{
		return $this->title;
	}

	/**
	 * Sets the title
	 *
	 * @param string $title
	 * @return void
	 */
	public function setTitle($title)
	{
		$this->title = $title;
	}

	/**
	 * Returns the preacher
	 *
	 * @return string $preacher
	 */
	public function getPreacher()
	{
		return $this->preacher;
	}

	/**
	 * Sets the preacher
	 *
	 * @param string $preacher
	 * @return void
	 */
	public function setPreacher($preacher)
	{
		$this->preacher = $preacher;
	}

	/**
	 * Returns the image
	 *
	 * @return \TYPO3\CMS\Core\Resource\FileReference
	 */
	public function getImage()
	{
		return $this->loadFileProperties($this->image);
	}

	/**
	 * Sets the image
	 *
	 * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $image
	 * @return void
	 */
	public function setImage(\TYPO3\CMS\Extbase\Domain\Model\FileReference $image)
	{
		$this->image = $image;
	}

	/**
	 * Returns the date
	 *
	 * @return \DateTime $date
	 */
	public function getDate($format = null)
	{
		if (!is_null($format)) {
			return $this->dateFormater($this->date, $format);
		} else {
			return $this->date;
		}
	}

	/**
	 * Sets the date
	 *
	 * @param string|\DateTime $date
	 * @return void
	 */
	public function setDate($date) {
		$date = \Lpc\LpcSermons\Utility\CastUtility::getDateObject($date);
		$this->date = $date;
	}


	/**
	 * Returns the description
	 *
	 * @return string $description
	 */
	public function getDescription()
	{
		return $this->description;
	}

	/**
	 * Sets the description
	 *
	 * @param string $description
	 * @return void
	 */
	public function setDescription($description)
	{
		$this->description = $description;
	}

	/**
	 * Returns the language
	 *
	 * @return string $language
	 */
	public function getLanguage()
	{
		return $this->language;
	}

	/**
	 * Sets the language
	 *
	 * @param string $language
	 * @return void
	 */
	public function setLanguage($language)
	{
		$this->language = $language;
	}

	/**
	 * __construct
	 */
	public function __construct()
	{
		//Do not remove the next line: It would break the functionality
		$this->initStorageObjects();
	}

	/**
	 * Initializes all ObjectStorage properties
	 * Do not modify this method!
	 * It will be rewritten on each save in the extension builder
	 * You may modify the constructor of this class instead
	 *
	 * @return void
	 */
	protected function initStorageObjects()
	{
		$this->resources = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
	}

	/**
	 * Returns the reference
	 *
	 * @return string $reference
	 */
	public function getReference()
	{
		return $this->reference;
	}

	/**
	 * Sets the reference
	 *
	 * @param string $reference
	 * @return void
	 */
	public function setReference($reference)
	{
		$this->reference = $reference;
	}

	/**
	 * Returns the url
	 *
	 * @return string $url
	 */
	public function getUrl()
	{
		return $this->url;
	}

	/**
	 * Sets the url
	 *
	 * @param string $url
	 * @return void
	 */
	public function setUrl($url)
	{
		$this->url = $url;
	}

	/**
	 * Returns the mp3file
	 *
	 *	maybe better \TYPO3\CMS\Extbase\Domain\Model\FileReference ?
	 * @return \TYPO3\CMS\Core\Resource\FileReference
	 */
	public function getMp3file()
	{
		$mp3file = $this->loadFileProperties($this->mp3file);
		return $mp3file;
	}

	/**
	 * Sets the mp3file
	 *
	 * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $mp3file
	 * @return void
	 */
	public function setMp3file(\TYPO3\CMS\Extbase\Domain\Model\FileReference $mp3file)
	{
		$this->mp3file = $mp3file;
	}

	/**
	 * Returns the podcastfile
	 *
	 * @return \TYPO3\CMS\Extbase\Domain\Model\FileReference
	 */
	public function getPodcastfile()
	{
		return $this->podcastfile;
	}

	/**
	 * Sets the podcastfile
	 *
	 * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $podcastfile
	 * @return void
	 */
	public function setPodcastfile(\TYPO3\CMS\Extbase\Domain\Model\FileReference $podcastfile)
	{
		$this->podcastfile = $podcastfile;
	}

	/**
	 * Returns the videourl
	 *
	 * @return string $videourl
	 */
	public function getVideourl()
	{
		return $this->videourl;
	}

	/**
	 * Sets the videourl
	 *
	 * @param string $videourl
	 * @return void
	 */
	public function setVideourl($videourl)
	{
		$this->videourl = $videourl;
	}

	/**
	 * Adds a FileReference
	 *
	 * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $resource
	 * @return void
	 */
	public function addResource(\TYPO3\CMS\Extbase\Domain\Model\FileReference $resource)
	{
		$this->resources->attach($resource);
	}

	/**
	 * Removes a FileReference
	 *
	 * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $resourceToRemove The FileReference to be removed
	 * @return void
	 */
	public function removeResource(\TYPO3\CMS\Extbase\Domain\Model\FileReference $resourceToRemove)
	{
		$this->resources->detach($resourceToRemove);
	}

	/**
	 * Returns the resources
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\FileReference> $resources
	 */
	public function getResources()
	{
		return $this->resources;
	}

	/**
	 * Sets the resources
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\FileReference> $resources
	 * @return void
	 */
	public function setResources(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $resources)
	{
		$this->resources = $resources;
	}

	/**
	 * Returns the podcast
	 *
	 * @return \Lpc\LpcSermons\Domain\Model\Podcast $podcast
	 */
	public function getPodcast()
	{
		return $this->podcast;
	}

	/**
	 * Sets the podcast
	 *
	 * @param \Lpc\LpcSermons\Domain\Model\Podcast $podcast
	 * @return void
	 */
	public function setPodcast(\Lpc\LpcSermons\Domain\Model\Podcast $podcast)
	{
		$this->podcast = $podcast;
	}

	/**
	 * @param array $cacheMp3info
	 */
	public function setCacheMp3info($cacheMp3info)
	{
		$this->cacheMp3info = json_encode($cacheMp3info);
	}

	/**
	 * @return array
	 */
	public function getCacheMp3info()
	{
		return json_decode($this->cacheMp3info, true);
	}

	/**
	 * @param int
	 */
	public function setCacheTs($cacheTs)
	{
		$this->cacheTs = $cacheTs;
	}

	/**
	 * @return int
	 */
	public function getCacheTs()
	{
		return $this->cacheTs;
	}


	/* ******************************************
	 *	Additionl mp3 file information
	 * ****************************************** */

	protected $filePropsAreLoaded = array();

	/**
	 * loads the original file information
	 * @param \TYPO3\CMS\Extbase\Domain\Model\FileReference $fileRef
	 * @return \TYPO3\CMS\Core\Resource\FileReference
	 */
	public function loadFileProperties($fileRef)
	{
		if (is_object($fileRef)) {
			if (!isset($this->filePropsAreLoaded[$fileRef->getUid()])) {
				$this->filePropsAreLoaded[$fileRef->getUid()] = true;
				if (!is_object($fileRef)) {
					return null;
				} elseif ($fileRef instanceof \TYPO3\CMS\Extbase\Persistence\Generic\LazyLoadingProxy) {
					$fileRef->_loadRealInstance();
				}
			}
			return $fileRef->getOriginalResource();
		}
	}

	/**
	 * Formtats a date
	 * @param $date
	 * @param $format
	 * @return mixed
	 */
	public function dateFormater($date, $format)
	{
		if (get_class($date) == 'DateTime') {
			return $date->format($format);
		}
		return '';
	}

	/**
	 * Media player code
	 * @var string
	 */
	protected $mp3fileAdditionalIsLoaded = false;

	/**
	 * Media player code
	 * @var string
	 */
	protected $mp3filePlayerCode = '';

	/**
	 * mp3 file size
	 * @var string
	 */
	protected $mp3fileSize = '';


	/**
	 * mp3 file duration
	 * @var string
	 */
	protected $mp3fileDuration = '';

	/**
	 * @param $mp3Info
	 */
	public function setMp3fileDuration($mp3Info)
	{
		$mp3fileDurationTemp = array();
		$mp3fileDurationTemp['playtime_seconds'] = $mp3Info['playtime_seconds'];
		$mp3fileDurationTemp['playtime_string'] = $mp3Info['playtime_string'];

		$playtimeParts = explode(':', $mp3Info['playtime_string']);
		$partMinutes = 0;
		$partSeconds = 0;
		if (count($playtimeParts) >= 1) {
			$partMinutes = intval($playtimeParts[0]);
			$partSeconds = intval($playtimeParts[1]);
		}
		$mp3fileDurationTemp['playtime_parts']['minutes'] = $partMinutes;
		$mp3fileDurationTemp['playtime_parts']['seconds'] = $partSeconds;

		$this->mp3fileDuration = $mp3fileDurationTemp;
	}

	/**
	 * @return string
	 */
	public function getMp3fileDuration()
	{
		$this->loadMp3Details();
		return $this->mp3fileDuration;
	}

	/**
	 * @param string $mp3fileSize
	 */
	public function setMp3fileSize($mp3fileSize)
	{
		$this->mp3fileSize = $mp3fileSize;
	}

	/**
	 * @return string
	 */
	public function getMp3fileSize()
	{
		$this->loadMp3Details();
		return $this->mp3fileSize;
	}

	/**
	 * @param $file
	 */
	public function setMp3filePlayerCode(\TYPO3\CMS\Core\Resource\FileReference $file)
	{
		$this->mp3filePlayerCode = $this->getPlayerCode($file);
	}

	/**
	 * @return \Lpc\LpcSermons\Domain\Model\Podcast
	 */
	public function getMp3filePlayerCode()
	{
		$this->loadMp3Details();
		return $this->mp3filePlayerCode;
	}

	/**
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 * @inject
	 */
	protected $configurationManager;

	/**
	 * @param $file
	 * @return string
	 */
	protected function getPlayerCode(\TYPO3\CMS\Core\Resource\FileReference $file)
	{
		$code = '';
		$mejsFlash = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extPath('lpc_sermons').'Resources/Public/MediaElement/build/flashmediaelement.swf';

		$settings = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);

		switch ($settings['flashPlayerType']) {
			case 'small':
				//91 only shows controls but no text - nice for table
				$width = 400;
				$height = 30;
				break;
			case 'big':
				$width = 400;
				$height = 170;
				break;
			default: //default, button
				$width = $height = 0; //actually 17, but then some browsers cut off some of the button...
				$code = '<a href="/" class="lpc_sermons-playBtn"></a>';
		}

		$mp3filePath = \Lpc\LpcSermons\Service\FileAbstractLayerService::getFalLinkByFileReference($file);

		// use naw_securedl if available
		if (ExtensionManagementUtility::isLoaded('naw_securedl')) {
			/** @var \Bitmotion\NawSecuredl\Service\SecureDownloadService $nawSecureDl */
			$nawSecureDl = GeneralUtility::makeInstance('Bitmotion\NawSecuredl\Service\SecureDownloadService');
			$mp3filePath = $nawSecureDl->publishResourceUri($mp3filePath);
		}

		$code .= '
            <audio class="mejs" width="' . $width . 'px" height="' . $height . 'px" controls="progress" preload="none" >
                <source type="audio/mp3" src="' . $mp3filePath . '" />
                <!-- Flash fallback for non-HTML5 browsers without JavaScript -->
                <object width="' . $width . 'px" height="' . $height . 'px" type="application/x-shockwave-flash" data="' . $mejsFlash . '">
                    <param name="movie" value="' . $mejsFlash . '" />
                  	<param name="flashvars" value="controls=true&file=' . $mp3filePath . '" />
                </object>
            </audio>
        ';

		return $code;
	}

	/**
	 * loads additional information about the mp3file
	 */
	public function loadMp3Details()
	{
		// load thhis information only once per request
		if (!$this->mp3fileAdditionalIsLoaded) {
			if (is_object($this->mp3file)) {
				$mp3file = $this->mp3file->getOriginalResource();

				if ($mp3file->getModificationTime() < $this->cacheTs) {
					//Use cached data
					$mp3Info = $this->getCacheMp3info();
				} else {
					// save information to cache
					$updateTime = time();

					// get getid3 from plugin lpcgetid3
					$getID3Service = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('Lpc\LpcGetID3\Service\GetID3Service');
					$getID3 = $getID3Service->getInstance();
					$mp3filePath = \Lpc\LpcSermons\Service\FileAbstractLayerService::getFalLinkByFileReference($mp3file);
					$mp3Info = $getID3->analyze(PATH_site.urldecode($mp3filePath));

					// copy cachable values
					$mp3FileTemp['playtime_seconds'] = $mp3Info['playtime_seconds'];
					$mp3FileTemp['playtime_string'] = $mp3Info['playtime_string'];

					// save to model (require an update on repository)
					$this->setCacheTs($updateTime);
					$this->setCacheMp3info($mp3FileTemp);
				}

				$this->setMp3fileDuration($mp3Info);
				$this->setMp3fileSize($mp3file->getSize());
				$this->setMp3filePlayerCode($mp3file);
			}
			$this->mp3fileAdditionalIsLoaded = true;
		}
	}

	/**
	 * @return int
	 */
	public function getCrdate() {
		return $this->crdate;
	}

	/**
	 * @param string|\DateTime $crdate
	 * @return void
	 */
	public function setCrdate($crdate) {
		$crdate = \Lpc\LpcSermons\Utility\CastUtility::getDateObject($crdate);

		$this->crdate = $crdate;
	}

	/**
	 * @return string
	 */
	public function getTstamp() {
		return $this->tstamp;
	}

	/**
	 * @param string $tstamp
	 */
	public function setTstamp($tstamp) {
		$this->tstamp = $tstamp;
	}

	/**
	 * @return string
	 */
	public function getHidden() {
		return $this->hidden;
	}

	/**
	 * @param string $hidden
	 */
	public function setHidden($hidden) {
		$this->hidden = $hidden;
	}

	/**
	 * @return string
	 */
	public function getDeleted() {
		return $this->deleted;
	}

	/**
	 * @param string $deleted
	 */
	public function setDeleted($deleted) {
		$this->deleted = $deleted;
	}

	/**
	 * @return string
	 */
	public function getCruserId() {
		return $this->cruserId;
	}

	/**
	 * @param string $cruserId
	 */
	public function setCruserId($cruserId) {
		$this->cruserId = $cruserId;
	}

	/**
	 * @return int
	 */
	public function getFeGroup() {
		return $this->feGroup;
	}

	/**
	 * @param int $feGroup
	 */
	public function setFeGroup($feGroup) {
		$this->feGroup = $feGroup;
	}

	/**
	 * @return int
	 */
	public function getKoolEventId() {
		return $this->koolEventId;
	}

	/**
	 * @param int $koolEventId
	 */
	public function setKoolEventId($koolEventId) {
		$this->koolEventId = $koolEventId;
	}

	/**
	 * @return \DateTime|null
	 */
	public function getStarttime() {
		return $this->starttime;
	}

	/**
	 * @param \DateTime|null|int|string $starttime
	 */
	public function setStarttime($starttime) {
		$starttime = \Lpc\LpcSermons\Utility\CastUtility::getDateObject($starttime);
		$this->starttime = $starttime;
	}

	/**
	 * @return \DateTime|null
	 */
	public function getEndtime() {
		return $this->endtime;
	}

	/**
	 * @param \DateTime|null|int|string $endtime
	 */
	public function setEndtime($endtime) {
		$endtime = \Lpc\LpcSermons\Utility\CastUtility::getDateObject($endtime);
		$this->endtime = $endtime;
	}



}