<?php
namespace Lpc\LpcSermons\Domain\Repository;


/***************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2014 Michael Hadorn <michael.hadorn@laupercomputing.com>, Lauper Computing
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
use TYPO3\CMS\Extbase\Persistence\QueryInterface;

/**
 * The repository for Sermons
 */
class SermonRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
{


	public function initializeObject()
	{

	}

	/**
	 * load sermons for frontend
	 * @param int $limit sql limit
	 * @param \Lpc\LpcSermons\Service\TypoScriptService $tsService
	 * @return \Lpc\LpcSermons\Domain\Model\Sermon[]
	 */
	public function loadForTable($limit = 0, $tsService)
	{
		$query = $this->createQuery();

		$filterPodcast = $tsService->getSetting('filterPodcast');

		$sortTs = $tsService->getSetting('sort');
		$sortOrderTs = $tsService->getSetting('sortOrder');

		$sort = 'date';
		$sortOrder = QueryInterface::ORDER_DESCENDING;

		if (strlen($sortTs) > 0) {
			$sort = $sortTs;
		}

		if (strlen($sortOrderTs) > 0 && strtolower($sortOrderTs) == 'asc') {
			$sortOrder = QueryInterface::ORDER_ASCENDING;
		}

		$query->setOrderings(array(
			$sort => $sortOrder
		));

		if (!empty($filterPodcast)) {
			$inPodcast = explode(',', $filterPodcast);

			$query->matching(
				$query->in('podcast', $inPodcast)
			);
		}

		$limit = intval($limit);
		if ($limit > 0) {
			$query->setLimit($limit);
		}

		$result = $query->execute();

		return $result;
	}

	/**
	 * get all sermons (doesn't respect storagepid)
	 * @param string $excludedPids it's possible to exclude some storagePids (comma-seperated list)
	 * @param int $lastImportDate created timestamp of last imported entry
	 * @return \Lpc\LpcSermons\Domain\Model\Sermon[]
	 */
	public function findAllExcludePids($excludedPids, $lastImportDate) {

		$excludedPidsClean = array();
		foreach (explode(',', $excludedPids) as $excludedPid) {
			$excludedPid = intval($excludedPid);
			if ($excludedPid > 0) {
				$excludedPidsClean[] = $excludedPid;
			}
		}

		$query = $this->createQuery();
		$query->getQuerySettings()->setRespectStoragePage(false);

		$filters = array();

		if (count($excludedPidsClean) > 0) {
			$filters[] = $query->logicalNot($query->in('pid', $excludedPidsClean));
		}
		if (intval($lastImportDate) > 0) {
			$filters[] = $query->greaterThan('crdate', $lastImportDate);
		}

		if (count($filters) > 0) {
			$query->matching($query->logicalAnd($filters));
		}

		return $query->execute();
	}

}