<?php

namespace Lpc\LpcSermons\Hook;

use Lpc\LpcSermons\Domain\Model\Podcast;
use Lpc\LpcSermons\Domain\Model\Sermon;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class PodcastXmlHook
 * @package Lpc\LpcSermons\Hook
 */
class PodcastXmlHook implements \TYPO3\CMS\Core\SingletonInterface
{

	/** @var \TYPO3\CMS\Extbase\Object\ObjectManager */
	protected $objectManager = null;

	/** @var \Lpc\LpcSermons\Domain\Repository\SermonRepository */
	protected $sermonRepository = null;

	/** @var \Lpc\LpcSermons\Domain\Repository\PodcastRepository */
	protected $podcastRepository = null;

	/** @var \Lpc\LpcSermons\Service\PodcastXmlService */
	protected $podcastXmlService = null;


	public function loadObjects() {
		$this->objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Object\\ObjectManager');
		$this->sermonRepository = $this->objectManager->get('Lpc\\LpcSermons\\Domain\\Repository\\SermonRepository');
		$this->podcastRepository = $this->objectManager->get('Lpc\\LpcSermons\\Domain\\Repository\\SermonRepository');
		$this->podcastXmlService = $this->objectManager->get('Lpc\\LpcSermons\\Service\\PodcastXmlService');
	}

	/**
	 * @param $status
	 * @param $table
	 * @param $id
	 * @param $fieldArray
	 * @param $_this
	 */
	public function processDatamap_afterDatabaseOperations($status, $table, $id, $fieldArray, $_this) {
		$podcastIds = array();

		/**
		 * Sermon has been added or edited
		 * Process only if new or update with changed fields for the sermons table
		 */
		if( $table == "tx_lpcsermons_domain_model_sermon" && ($status == "new" || ($status == "update" && sizeof($fieldArray) > 0)) ) {
			$this->loadObjects();

			if($status == "new") {
				$podcastId[] = $fieldArray["podcast_id"];
			} else {
				/** @var Sermon $sermon */
				$sermon = $this->sermonRepository->findByUid($id);
				// previous podcast
				$podcast = $sermon->getPodcast();
				if (!is_null($podcast)) {
					$podcastIds[] = $podcast->getUid();
					if(isset($fieldArray["podcast"])) {
						if (intval($fieldArray["podcast"]) > 0) {
							// also add the new podcast to generate
							$podcastIds[] = intval($fieldArray["podcast"]);
						}
					}
				}
			}
		}

		/**
		 * Podcast has been saved, so write XML file
		 */
		if($table == "tx_lpcsermons_domain_model_podcast" && ($status == "update" && sizeof($fieldArray) > 0)) {
			if(!$id) return FALSE;

			$this->loadObjects();
			$podcastIds[] = $id;
		}

		if(!empty($podcastIds)) {
			$this->generatePodcasts($podcastIds);
		}

	}

	/**
	 *
	 * @param array $podcastIds
	 * @return bool
	 */
	public function generatePodcasts($podcastIds) {
		foreach ($podcastIds as $podcastId) {
			if (intval($podcastId) > 0) {
				$this->podcastXmlService->generatePodcast($podcastId);
			}
		}
		return false;
	}

	/**
	 *
	 * function processCmdmap_postProcess($command, $table, $id, $value, $_this) {
	 * Hook if Sermon or Podcast has been deleted
	 *
	 * @param $command
	 * @param $table
	 * @param $id
	 * @param $value
	 * @param $_this
	 */
	public function processCmdmap_postProcess($command, $table, $id, $value, $_this) {
		if($command != 'delete') {
			return;
		}

		// delete the xml file if the podcast has been deleted
		if($table == 'tx_lpcsermons_domain_model_podcast') {
			$this->loadObjects();

			/** @var Podcast $podcast */
			$podcasts = $this->podcastRepository->findByUid($id);
			foreach ($podcasts as $podcast) {
				$path = GeneralUtility::getIndpEnv('TYPO3_DOCUMENT_ROOT').'/'.$podcast->getXmlFile();
				if(GeneralUtility::validPathStr($path)) {
					unlink($path);
				}
			}

			$res = $GLOBALS["TYPO3_DB"]->exec_SELECTquery("xml_file", "tx_lpcsermons_domain_model_podcast", "`uid` = ".$id);
			if($GLOBALS["TYPO3_DB"]->sql_num_rows($res) == 1) {
				$row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res);
				$path = GeneralUtility::getIndpEnv('TYPO3_DOCUMENT_ROOT').'/'.$row['xml_file'];

				if(GeneralUtility::validPathStr($path)) {
					unlink($path);
				}
			}
		}

		// update the xml file if a sermon has been deleted
		if($table == 'tx_lpcsermons_domain_model_sermon') {
			$this->loadObjects();

			$podcastIds = array();
			$sermon = $this->sermonRepository->findByUid($id);
			if (!is_null($sermon)) {
				if (!is_null($sermon->getPodcast())) {
					$podcastIds[] = $sermon->getPodcast()->getUid();
					$this->generatePodcasts($podcastIds);
				}
			}
		}
	}

}