<?php
namespace Lpc\LpcSermons\Service;


use Lpc\LpcSermons\Domain\Model\Podcast;
use Lpc\LpcSermons\Domain\Model\Sermon;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Domain\Model\FileReference;

class PodcastXmlService implements \TYPO3\CMS\Core\SingletonInterface
{

	/**
	 * sermonRepository
	 *
	 * @var \Lpc\LpcSermons\Domain\Repository\SermonRepository
	 * @inject
	 */
	protected $sermonRepository;

	/**
	 * podcastRepository
	 *
	 * @var \Lpc\LpcSermons\Domain\Repository\PodcastRepository
	 * @inject
	 */
	protected $podcastRepository;

	/**
	 * @var \Lpc\LpcSermons\Service\TypoScriptService
	 * @inject
	 */
	protected $typoScriptService;

	/**
	 * generates the xml for the podcast
	 * @param $podcastId
	 */
	function generatePodcast($podcastId)
	{
		if (!$podcastId) return FALSE;

		/** @var Podcast $podcast */
		$podcast = $this->podcastRepository->findByUid($podcastId);

		// Get sermons for this podcast (ignore storage page)
		$querySettings = $this->sermonRepository->createQuery()->getQuerySettings();
		$querySettings->setRespectStoragePage(false);
		$this->sermonRepository->setDefaultQuerySettings($querySettings);
		$sermons = $this->sermonRepository->findByPodcast($podcastId);

		$websiteUrl = self::getWebsiteUrl($podcast->getPid());

		//Generate XML
		$xml = '<?xml version="1.0" encoding="UTF-8"?>' . "\n";
		$xml .= '<rss xmlns:itunes="http://www.itunes.com/dtds/podcast-1.0.dtd" version="2.0">' . "\n";
		$xml .= '<channel>' . "\n";
		$xml .= "\t" . '<title>' . htmlspecialchars($podcast->getTitle()) . '</title>' . "\n";
		$xml .= "\t" . '<link>' . $websiteUrl . '</link>' . "\n";
		$xml .= "\t" . '<language>' . htmlspecialchars($podcast->getLanguage()) . '</language>' . "\n";
		$xml .= "\t" . '<copyright>' . htmlspecialchars($podcast->getCopyright()) . '</copyright>' . "\n";
		$xml .= "\t" . '<itunes:subtitle>' . htmlspecialchars($podcast->getSubtitle()) . '</itunes:subtitle>' . "\n";
		$xml .= "\t" . '<itunes:author>' . htmlspecialchars($podcast->getAuthor()) . '</itunes:author>' . "\n";
		$xml .= "\t" . '<itunes:summary>' . htmlspecialchars($podcast->getDescription()) . '</itunes:summary>' . "\n";
		$xml .= "\t" . '<description>' . htmlspecialchars($podcast->getDescription()) . '</description>' . "\n";

		if ($podcast->getExplicit()) {
			$xml .= "\t" . '<itunes:explicit>' . htmlspecialchars($podcast->getExplicit()) . '</itunes:explicit>' . "\n";
		}

		if ($podcast->getEditorEmail()) {
			$xml .= "\t" . '<managingEditor>' . htmlspecialchars($podcast->getEditorEmail()) . ' (' . htmlspecialchars($podcast->getEditorName()) . ')</managingEditor>' . "\n";
			$xml .= "\t" . '<itunes:owner>' . "\n";
			if ($podcast->getEditorName()) $xml .= "\t\t" . '<itunes:name>' . htmlspecialchars($podcast->getEditorName()) . '</itunes:name>' . "\n";
			$xml .= "\t\t" . '<itunes:email>' . htmlspecialchars($podcast->getEditorEmail()) . '</itunes:email>' . "\n";
			$xml .= "\t" . '</itunes:owner>' . "\n";
		}

		if ($podcast->getCategory()) {
			$categoryArr = explode(',', $podcast->getCategory());
			foreach ($categoryArr as $categoryFull) {
				$categoryParts = explode(' > ', $categoryFull);
				// set main category
				$xml .= "\t" . '<itunes:category text="' . htmlspecialchars($categoryParts["0"]) . '"';
				//set subcategory if exist otherwise close tag
				if ($categoryParts["1"]) {
					$xml .= '>' . "\n\t\t" . '<itunes:category text="' . htmlspecialchars($categoryParts["1"]) . '" />' . "\n\t" . '</itunes:category>' . "\n";
				} else {
					$xml .= ' />' . "\n";
				}
			}
		}

		/** @var FileReference $podcastImage */
		$podcastImage = $podcast->getImage();
		if ($podcastImage) {
			$imagePath = \Lpc\LpcSermons\Service\FileAbstractLayerService::getFalLinkByFileReference($podcastImage);
			if(substr($imagePath, 0, 1) == '/') $imagePath = substr($imagePath, 1);
			$imagePathFull = $websiteUrl . $imagePath;
			$xml .= "\t" . '<image>' . "\n";
			$xml .= "\t\t" . '<url>' . $imagePathFull . '</url>' . "\n";
			$xml .= "\t\t" . '<title>' . htmlspecialchars($podcast->getTitle()) . '</title>' . "\n";
			$xml .= "\t\t" . '<link>' . $websiteUrl . '</link>' . "\n";
			$xml .= "\t" . '</image>' . "\n";
			$xml .= "\t" . '<itunes:image href="' . $imagePathFull . '" />' . "\n";
		}
		$xml .= "\t" . '<lastBuildDate>' . date('D, d M Y H:i:s O', time()) . '</lastBuildDate>' . "\n";
		$xml .= "\t" . '<pubDate>' . date_format($podcast->getCrdate(), 'D, d M Y H:i:s O') . '</pubDate>' . "\n";
		$xml .= "\t" . '<generator>TYPO3 extension lps_sermons</generator>' . "\n";

		// Add sermons
		foreach ($sermons as $sermon) {
			$xml .= self::addItem($sermon, $podcast, $websiteUrl);
		}

		$xml .= '</channel>' . "\n";
		$xml .= '</rss>' . "\n";

		//convert the file content to utf 8 if its another charset
		//TODO Check this...
		if ($GLOBALS['LANG']) {
			$xml = $GLOBALS['LANG']->csConvObj->utf8_encode($xml, $GLOBALS['LANG']->charSet);
		} else {
			// if $GLOBALS['LANG'] is not initialised (Frontend), make a own instance
			require_once(PATH_typo3 . 'sysext/lang/lang.php');
			$LANG = t3lib_div::makeInstance('language');
			$LANG->init($GLOBALS['TSFE']->config['config']['language']);
			$LANG->csConvObj->utf8_encode($xml, $LANG->charSet);
		}

		self::writeToFile($podcast->getXmlFile(), $xml);
	}


	/**
	 * Adds a sermon to the xml
	 *
	 * @param $sermon
	 * @param $podcast
	 * @param $websiteUrl
	 * @return string
	 */
	function addItem(Sermon $sermon, Podcast $podcast, $websiteUrl)
	{
		$xml = '';
		$files = array();

		//Allowed file types
		$file_types = array('mp3' => 'audio/mpeg', 'm4a' => 'audio/x-m4a', 'mp4' => 'video/mp4', 'm4v' => 'video/x-m4v', 'mov' => 'video/quicktime', 'pdf' => 'application/pdf');

		//Get mp3 file
		$getByFileType = '';
		if ($sermon->getPodcastfile()) {
			$getByFileType = 'podcastfile';
		} else if ($sermon->getMp3file()) {
			$getByFileType = 'mp3file';
		}
		if (strlen($getByFileType) > 0) {
			$files[] = \Lpc\LpcSermons\Service\FileAbstractLayerService::getFalLinkByUid($sermon->getUid(), $getByFileType, 'tx_lpcsermons_domain_model_sermon', true);
		}

		//Add media files from resources
		if (!$podcast->getExcludeResources() && $sermon->getResources()) {
			$uploads = \Lpc\LpcSermons\Service\FileAbstractLayerService::getFalLinkByUid($sermon->getUid(), 'resources', 'tx_lpcsermons_domain_model_sermon', false);
			foreach ($uploads as $upload) {
				if (in_array(strtolower(substr($upload, -3)), array_keys($file_types))) {
					$files[] = trim($upload);
				}
			}
		}

		//Add an item for each file
		foreach ($files as $file) {
			$file = trim($file);
			$type = strtolower(substr($file, -3));
			if (!in_array($type, array_keys($file_types))) continue;

			//Check for relative or absolute path
			if (substr($file, 0, 7) == 'http://' || substr($file, 0, 8) == 'https://') {
				$file_url = $file;
			} else {
				//Remove first slash because websiteUrl always ends with a slash
				if(substr($file, 0, 1) == '/') $file = substr($file, 1);
				$file_url = $websiteUrl . $file;
			}

			// be sure, to have mp3 infos
			$sermon->loadMp3Details();

			$mp3FileSize = $sermon->getMp3fileSize();
			$mp3FileDuration = $sermon->getMp3fileDuration();

			$xml .= "\n\t" . '<item>' . "\n";

			$xml .= "\t\t" . '<title>' . htmlspecialchars($sermon->getTitle()) . '</title>' . "\n";
			$xml .= "\t\t" . '<author>' . htmlspecialchars($podcast->getEditorEmail()) . ' (' . htmlspecialchars($sermon->getPreacher()) . ')</author>' . "\n";
			$xml .= "\t\t" . '<itunes:author>' . htmlspecialchars($sermon->getPreacher()) . '</itunes:author>' . "\n";
			$xml .= "\t\t" . '<description>' . htmlspecialchars($sermon->getDescription()) . '</description>' . "\n";
			$xml .= "\t\t" . '<itunes:summary>' . htmlspecialchars($sermon->getDescription()) . '</itunes:summary>' . "\n";
			if ($type != 'mp3' && in_array($type, array_keys($file_types))) {
				if (substr($file, 0, 9) == 'fileadmin') {  //HACK for vinDACH
					$file_url = $websiteUrl . $file;
				}
				$xml .= "\t\t" . '<enclosure url="' . $file_url . '" type="' . $file_types[$type] . '" />' . "\n";
			} else {
				$xml .= "\t\t" . '<enclosure url="' . $file_url . '"' . (intval($mp3FileSize) > 0 ? (' length="' . $mp3FileSize . '"') : '') . ' type="' . $file_types[$type] . '" />' . "\n";
				if (strlen($mp3FileDuration['playtime_string']) > 0) $xml .= "\t\t" . '<itunes:duration>' . $mp3FileDuration['playtime_string'] . '</itunes:duration>' . "\n";
			}
			$xml .= "\t\t" . '<guid>' . $file_url . '</guid>' . "\n";
			$xml .= "\t\t" . '<pubDate>' . $sermon->getDate('D, d M Y H:i:s O') . '</pubDate>' . "\n";

			$xml .= "\t" . '</item>' . "\n";
		}
		return $xml;
	}


	function writeToFile($file, $xml)
	{
		$path = GeneralUtility::getIndpEnv('TYPO3_DOCUMENT_ROOT') . '/' . $file;
		if (GeneralUtility::validPathStr($path)) {
			return GeneralUtility::writeFile($path, $xml);
		}
		return false;
	}


	function getWebsiteUrl($pageId = 1)
	{
		// parse the ts setup code because is not loaded in backend to get the baseurl
		$sysPageObj = GeneralUtility::makeInstance('\TYPO3\CMS\Frontend\Page\PageRepository');
		$rootLine = $sysPageObj->getRootLine($pageId);
		$TSObj = GeneralUtility::makeInstance('\TYPO3\CMS\Core\TypoScript\ExtendedTemplateService');
		$TSObj->tt_track = 0;
		$TSObj->init();
		$TSObj->runThroughTemplates($rootLine);
		$TSObj->generateConfig();

		// use the baseurl if it is set instead of the environment var (fix problems in portal solutions)
		if ($TSObj->setup['config.']['baseURL']) {
			$retWebsiteUrl = $TSObj->setup['config.']['baseURL'];
		} else {
			$retWebsiteUrl = GeneralUtility::getIndpEnv('TYPO3_SITE_URL');
		}

		//Add trailing slash if not set
		$retWebsiteUrl .= substr($retWebsiteUrl, -1) != '/' ? '/' : '';

		return $retWebsiteUrl;
	}


}
