<?php

namespace Lpc\LpcSermons\Tests\Unit\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Michael Hadorn <michael.hadorn@laupercomputing.ch>, Lauper Computing
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Test case for class \Lpc\LpcSermons\Domain\Model\Sermon.
 *
 * @copyright Copyright belongs to the respective authors
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 * @author Michael Hadorn <michael.hadorn@laupercomputing.ch>
 */
class SermonTest extends \TYPO3\CMS\Core\Tests\UnitTestCase {
	/**
	 * @var \Lpc\LpcSermons\Domain\Model\Sermon
	 */
	protected $subject = NULL;

	protected function setUp() {
		$this->subject = new \Lpc\LpcSermons\Domain\Model\Sermon();
	}

	protected function tearDown() {
		unset($this->subject);
	}

	/**
	 * @test
	 */
	public function getTitleReturnsInitialValueForString() {
		$this->assertSame(
			'',
			$this->subject->getTitle()
		);
	}

	/**
	 * @test
	 */
	public function setTitleForStringSetsTitle() {
		$this->subject->setTitle('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'title',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getPreacherReturnsInitialValueForString() {
		$this->assertSame(
			'',
			$this->subject->getPreacher()
		);
	}

	/**
	 * @test
	 */
	public function setPreacherForStringSetsPreacher() {
		$this->subject->setPreacher('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'preacher',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getImageReturnsInitialValueForFileReference() {
		$this->assertEquals(
			NULL,
			$this->subject->getImage()
		);
	}

	/**
	 * @test
	 */
	public function setImageForFileReferenceSetsImage() {
		$fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
		$this->subject->setImage($fileReferenceFixture);

		$this->assertAttributeEquals(
			$fileReferenceFixture,
			'image',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getDateReturnsInitialValueForDateTime() {
		$this->assertEquals(
			NULL,
			$this->subject->getDate()
		);
	}

	/**
	 * @test
	 */
	public function setDateForDateTimeSetsDate() {
		$dateTimeFixture = new \DateTime();
		$this->subject->setDate($dateTimeFixture);

		$this->assertAttributeEquals(
			$dateTimeFixture,
			'date',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getDescriptionReturnsInitialValueForString() {
		$this->assertSame(
			'',
			$this->subject->getDescription()
		);
	}

	/**
	 * @test
	 */
	public function setDescriptionForStringSetsDescription() {
		$this->subject->setDescription('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'description',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getLanguageReturnsInitialValueForString() {
		$this->assertSame(
			'',
			$this->subject->getLanguage()
		);
	}

	/**
	 * @test
	 */
	public function setLanguageForStringSetsLanguage() {
		$this->subject->setLanguage('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'language',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getReferenceReturnsInitialValueForString() {
		$this->assertSame(
			'',
			$this->subject->getReference()
		);
	}

	/**
	 * @test
	 */
	public function setReferenceForStringSetsReference() {
		$this->subject->setReference('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'reference',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getUrlReturnsInitialValueForString() {
		$this->assertSame(
			'',
			$this->subject->getUrl()
		);
	}

	/**
	 * @test
	 */
	public function setUrlForStringSetsUrl() {
		$this->subject->setUrl('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'url',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getMp3fileReturnsInitialValueForFileReference() {
		$this->assertEquals(
			NULL,
			$this->subject->getMp3file()
		);
	}

	/**
	 * @test
	 */
	public function setMp3fileForFileReferenceSetsMp3file() {
		$fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
		$this->subject->setMp3file($fileReferenceFixture);

		$this->assertAttributeEquals(
			$fileReferenceFixture,
			'mp3file',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getPodcastfileReturnsInitialValueForFileReference() {
		$this->assertEquals(
			NULL,
			$this->subject->getPodcastfile()
		);
	}

	/**
	 * @test
	 */
	public function setPodcastfileForFileReferenceSetsPodcastfile() {
		$fileReferenceFixture = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
		$this->subject->setPodcastfile($fileReferenceFixture);

		$this->assertAttributeEquals(
			$fileReferenceFixture,
			'podcastfile',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getVideourlReturnsInitialValueForString() {
		$this->assertSame(
			'',
			$this->subject->getVideourl()
		);
	}

	/**
	 * @test
	 */
	public function setVideourlForStringSetsVideourl() {
		$this->subject->setVideourl('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'videourl',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getResourcesReturnsInitialValueForFileReference() {
		$newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$this->assertEquals(
			$newObjectStorage,
			$this->subject->getResources()
		);
	}

	/**
	 * @test
	 */
	public function setResourcesForFileReferenceSetsResources() {
		$resource = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
		$objectStorageHoldingExactlyOneResources = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$objectStorageHoldingExactlyOneResources->attach($resource);
		$this->subject->setResources($objectStorageHoldingExactlyOneResources);

		$this->assertAttributeEquals(
			$objectStorageHoldingExactlyOneResources,
			'resources',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function addResourceToObjectStorageHoldingResources() {
		$resource = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
		$resourcesObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('attach'), array(), '', FALSE);
		$resourcesObjectStorageMock->expects($this->once())->method('attach')->with($this->equalTo($resource));
		$this->inject($this->subject, 'resources', $resourcesObjectStorageMock);

		$this->subject->addResource($resource);
	}

	/**
	 * @test
	 */
	public function removeResourceFromObjectStorageHoldingResources() {
		$resource = new \TYPO3\CMS\Extbase\Domain\Model\FileReference();
		$resourcesObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('detach'), array(), '', FALSE);
		$resourcesObjectStorageMock->expects($this->once())->method('detach')->with($this->equalTo($resource));
		$this->inject($this->subject, 'resources', $resourcesObjectStorageMock);

		$this->subject->removeResource($resource);

	}

	/**
	 * @test
	 */
	public function getPodcastReturnsInitialValueForPodcast() {
		$this->assertEquals(
			NULL,
			$this->subject->getPodcast()
		);
	}

	/**
	 * @test
	 */
	public function setPodcastForPodcastSetsPodcast() {
		$podcastFixture = new \Lpc\LpcSermons\Domain\Model\Podcast();
		$this->subject->setPodcast($podcastFixture);

		$this->assertAttributeEquals(
			$podcastFixture,
			'podcast',
			$this->subject
		);
	}
}
